function y = fft19(x,u,ip,op)
% y = fft19(x,u,ip,op)
% y  : the 19 point DFT of x 
% u  : a vector of precomputed multiplicative constants
% ip : input permutation
% op : ouput permutation

y = zeros(19,1);
x = x(ip);                                        % input permutation
x(2:19) = KRED([2,3],[1,2],2,x(2:19));            % reduction operations
y(1) = x(1)+x(2);                                 % DC term calculation
% -------------------- block : 1 -------------------------------------------------
y(2) = x(2)*u(1);                       
% -------------------- block : 2 -------------------------------------------------
y(3) = x(3)*u(2);                       
% -------------------- block : 3 -------------------------------------------------
v = ID2I(1,1,x(4:5));              % v = (I(1) kron D2 kron I(1)) * x(4:5)
v = v.*u(3:5);                          
y(4:5) = ID2tI(1,1,v);             % y(4:5) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 6 = 2 * 3 -----------------------------------------
v = ID2I(1,1,x(6:7));              % v = (I(1) kron D2 kron I(1)) * x(6:7)
v = v.*u(6:8);                          
y(6:7) = ID2tI(1,1,v);             % y(6:7) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 9 -------------------------------------------------
v = ID3I(2,1,x(8:13));             % v = (I(2) kron D3 kron I(1)) * x(8:13)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(9:23);                         
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(8:13) = ID3tI(2,1,v);            % y(8:13) = (I(2) kron D3' kron I(1)) * v
% -------------------- block : 18 = 2 * 9 ----------------------------------------
v = ID3I(2,1,x(14:19));            % v = (I(2) kron D3 kron I(1)) * x(14:19)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(24:38);                        
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(14:19) = ID3tI(2,1,v);           % y(14:19) = (I(2) kron D3' kron I(1)) * v
% --------------------------------------------------------------------------------
y(2) = y(1)+y(2);                                 % DC term calculation
y(2:19) = tKRED([2,3],[1,2],2,y(2:19));           % transpose reduction operations
y = y(op);                                        % output permutation

% For complex data - 
% Total Number of Real Multiplications : 76
% Total Number of Real Additions: 404

