function y = fft37(x,u,ip,op)
% y = fft37(x,u,ip,op)
% y  : the 37 point DFT of x 
% u  : a vector of precomputed multiplicative constants
% ip : input permutation
% op : ouput permutation

y = zeros(37,1);
x = x(ip);                                        % input permutation
x(2:37) = KRED([2,3],[2,2],2,x(2:37));            % reduction operations
y(1) = x(1)+x(2);                                 % DC term calculation
% -------------------- block : 1 -------------------------------------------------
y(2) = x(2)*u(1);                       
% -------------------- block : 2 -------------------------------------------------
y(3) = x(3)*u(2);                       
% -------------------- block : 4 -------------------------------------------------
v = ID2I(1,1,x(4:5));              % v = (I(1) kron D2 kron I(1)) * x(4:5)
v = v.*u(3:5);                          
y(4:5) = ID2tI(1,1,v);             % y(4:5) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 3 -------------------------------------------------
v = ID2I(1,1,x(6:7));              % v = (I(1) kron D2 kron I(1)) * x(6:7)
v = v.*u(6:8);                          
y(6:7) = ID2tI(1,1,v);             % y(6:7) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 6 = 2 * 3 -----------------------------------------
v = ID2I(1,1,x(8:9));              % v = (I(1) kron D2 kron I(1)) * x(8:9)
v = v.*u(9:11);                         
y(8:9) = ID2tI(1,1,v);             % y(8:9) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 12 = 4 * 3 ----------------------------------------
v = ID2I(1,2,x(10:13));            % v = (I(1) kron D2 kron I(2)) * x(10:13)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(12:20);                        
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(10:13) = ID2tI(2,1,v);           % y(10:13) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 9 -------------------------------------------------
v = ID3I(2,1,x(14:19));            % v = (I(2) kron D3 kron I(1)) * x(14:19)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(21:35);                        
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(14:19) = ID3tI(2,1,v);           % y(14:19) = (I(2) kron D3' kron I(1)) * v
% -------------------- block : 18 = 2 * 9 ----------------------------------------
v = ID3I(2,1,x(20:25));            % v = (I(2) kron D3 kron I(1)) * x(20:25)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(36:50);                        
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(20:25) = ID3tI(2,1,v);           % y(20:25) = (I(2) kron D3' kron I(1)) * v
% -------------------- block : 36 = 4 * 9 ----------------------------------------
v = ID3I(4,1,x(26:37));            % v = (I(4) kron D3 kron I(1)) * x(26:37)
v = ID2I(1,10,v);                  % v = (I(1) kron D2 kron I(10)) * v
v = ID2I(3,5,v);                   % v = (I(3) kron D2 kron I(5)) * v
v = v.*u(51:95);                        
v = ID2tI(1,15,v);                 % v = (I(1) kron D2' kron I(15)) * v
v = ID2tI(2,5,v);                  % v = (I(2) kron D2' kron I(5)) * v
y(26:37) = ID3tI(4,1,v);           % y(26:37) = (I(4) kron D3' kron I(1)) * v
% --------------------------------------------------------------------------------
y(2) = y(1)+y(2);                                 % DC term calculation
y(2:37) = tKRED([2,3],[2,2],2,y(2:37));           % transpose reduction operations
y = y(op);                                        % output permutation

% For complex data - 
% Total Number of Real Multiplications : 190
% Total Number of Real Additions: 990

