function y = fft41(x,u,ip,op)
% y = fft41(x,u,ip,op)
% y  : the 41 point DFT of x 
% u  : a vector of precomputed multiplicative constants
% ip : input permutation
% op : ouput permutation

y = zeros(41,1);
x = x(ip);                                        % input permutation
x(2:41) = KRED([2,5],[3,1],2,x(2:41));            % reduction operations
y(1) = x(1)+x(2);                                 % DC term calculation
% -------------------- block : 1 -------------------------------------------------
y(2) = x(2)*u(1);                       
% -------------------- block : 2 -------------------------------------------------
y(3) = x(3)*u(2);                       
% -------------------- block : 4 -------------------------------------------------
v = ID2I(1,1,x(4:5));              % v = (I(1) kron D2 kron I(1)) * x(4:5)
v = v.*u(3:5);                          
y(4:5) = ID2tI(1,1,v);             % y(4:5) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 8 -------------------------------------------------
v = ID2I(1,2,x(6:9));              % v = (I(1) kron D2 kron I(2)) * x(6:9)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(6:14);                         
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(6:9) = ID2tI(2,1,v);             % y(6:9) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 5 -------------------------------------------------
v = ID2I(1,2,x(10:13));            % v = (I(1) kron D2 kron I(2)) * x(10:13)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(15:23);                        
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(10:13) = ID2tI(2,1,v);           % y(10:13) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 10 = 2 * 5 ----------------------------------------
v = ID2I(1,2,x(14:17));            % v = (I(1) kron D2 kron I(2)) * x(14:17)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(24:32);                        
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(14:17) = ID2tI(2,1,v);           % y(14:17) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 20 = 4 * 5 ----------------------------------------
v = ID2I(1,4,x(18:25));            % v = (I(1) kron D2 kron I(4)) * x(18:25)
v = ID2I(3,2,v);                   % v = (I(3) kron D2 kron I(2)) * v
v = ID2I(9,1,v);                   % v = (I(9) kron D2 kron I(1)) * v
v = v.*u(33:59);                        
v = ID2tI(1,9,v);                  % v = (I(1) kron D2' kron I(9)) * v
v = ID2tI(2,3,v);                  % v = (I(2) kron D2' kron I(3)) * v
y(18:25) = ID2tI(4,1,v);           % y(18:25) = (I(4) kron D2' kron I(1)) * v
% -------------------- block : 40 = 8 * 5 ----------------------------------------
v = ID2I(1,8,x(26:41));            % v = (I(1) kron D2 kron I(8)) * x(26:41)
v = ID2I(3,4,v);                   % v = (I(3) kron D2 kron I(4)) * v
v = ID2I(9,2,v);                   % v = (I(9) kron D2 kron I(2)) * v
v = ID2I(27,1,v);                  % v = (I(27) kron D2 kron I(1)) * v
v = v.*u(60:140);                       
v = ID2tI(1,27,v);                 % v = (I(1) kron D2' kron I(27)) * v
v = ID2tI(2,9,v);                  % v = (I(2) kron D2' kron I(9)) * v
v = ID2tI(4,3,v);                  % v = (I(4) kron D2' kron I(3)) * v
y(26:41) = ID2tI(8,1,v);           % y(26:41) = (I(8) kron D2' kron I(1)) * v
% --------------------------------------------------------------------------------
y(2) = y(1)+y(2);                                 % DC term calculation
y(2:41) = tKRED([2,5],[3,1],2,y(2:41));           % transpose reduction operations
y = y(op);                                        % output permutation

% For complex data - 
% Total Number of Real Multiplications : 280
% Total Number of Real Additions: 1140

