function y = fft73(x,u,ip,op)
% y = fft73(x,u,ip,op)
% y  : the 73 point DFT of x 
% u  : a vector of precomputed multiplicative constants
% ip : input permutation
% op : ouput permutation

y = zeros(73,1);
x = x(ip);                                        % input permutation
x(2:73) = KRED([2,3],[3,2],2,x(2:73));            % reduction operations
y(1) = x(1)+x(2);                                 % DC term calculation
% -------------------- block : 1 -------------------------------------------------
y(2) = x(2)*u(1);                       
% -------------------- block : 2 -------------------------------------------------
y(3) = x(3)*u(2);                       
% -------------------- block : 4 -------------------------------------------------
v = ID2I(1,1,x(4:5));              % v = (I(1) kron D2 kron I(1)) * x(4:5)
v = v.*u(3:5);                          
y(4:5) = ID2tI(1,1,v);             % y(4:5) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 8 -------------------------------------------------
v = ID2I(1,2,x(6:9));              % v = (I(1) kron D2 kron I(2)) * x(6:9)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(6:14);                         
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(6:9) = ID2tI(2,1,v);             % y(6:9) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 3 -------------------------------------------------
v = ID2I(1,1,x(10:11));            % v = (I(1) kron D2 kron I(1)) * x(10:11)
v = v.*u(15:17);                        
y(10:11) = ID2tI(1,1,v);           % y(10:11) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 6 = 2 * 3 -----------------------------------------
v = ID2I(1,1,x(12:13));            % v = (I(1) kron D2 kron I(1)) * x(12:13)
v = v.*u(18:20);                        
y(12:13) = ID2tI(1,1,v);           % y(12:13) = (I(1) kron D2' kron I(1)) * v
% -------------------- block : 12 = 4 * 3 ----------------------------------------
v = ID2I(1,2,x(14:17));            % v = (I(1) kron D2 kron I(2)) * x(14:17)
v = ID2I(3,1,v);                   % v = (I(3) kron D2 kron I(1)) * v
v = v.*u(21:29);                        
v = ID2tI(1,3,v);                  % v = (I(1) kron D2' kron I(3)) * v
y(14:17) = ID2tI(2,1,v);           % y(14:17) = (I(2) kron D2' kron I(1)) * v
% -------------------- block : 24 = 8 * 3 ----------------------------------------
v = ID2I(1,4,x(18:25));            % v = (I(1) kron D2 kron I(4)) * x(18:25)
v = ID2I(3,2,v);                   % v = (I(3) kron D2 kron I(2)) * v
v = ID2I(9,1,v);                   % v = (I(9) kron D2 kron I(1)) * v
v = v.*u(30:56);                        
v = ID2tI(1,9,v);                  % v = (I(1) kron D2' kron I(9)) * v
v = ID2tI(2,3,v);                  % v = (I(2) kron D2' kron I(3)) * v
y(18:25) = ID2tI(4,1,v);           % y(18:25) = (I(4) kron D2' kron I(1)) * v
% -------------------- block : 9 -------------------------------------------------
v = ID3I(2,1,x(26:31));            % v = (I(2) kron D3 kron I(1)) * x(26:31)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(57:71);                        
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(26:31) = ID3tI(2,1,v);           % y(26:31) = (I(2) kron D3' kron I(1)) * v
% -------------------- block : 18 = 2 * 9 ----------------------------------------
v = ID3I(2,1,x(32:37));            % v = (I(2) kron D3 kron I(1)) * x(32:37)
v = ID2I(1,5,v);                   % v = (I(1) kron D2 kron I(5)) * v
v = v.*u(72:86);                        
v = ID2tI(1,5,v);                  % v = (I(1) kron D2' kron I(5)) * v
y(32:37) = ID3tI(2,1,v);           % y(32:37) = (I(2) kron D3' kron I(1)) * v
% -------------------- block : 36 = 4 * 9 ----------------------------------------
v = ID3I(4,1,x(38:49));            % v = (I(4) kron D3 kron I(1)) * x(38:49)
v = ID2I(1,10,v);                  % v = (I(1) kron D2 kron I(10)) * v
v = ID2I(3,5,v);                   % v = (I(3) kron D2 kron I(5)) * v
v = v.*u(87:131);                       
v = ID2tI(1,15,v);                 % v = (I(1) kron D2' kron I(15)) * v
v = ID2tI(2,5,v);                  % v = (I(2) kron D2' kron I(5)) * v
y(38:49) = ID3tI(4,1,v);           % y(38:49) = (I(4) kron D3' kron I(1)) * v
% -------------------- block : 72 = 8 * 9 ----------------------------------------
v = ID3I(8,1,x(50:73));            % v = (I(8) kron D3 kron I(1)) * x(50:73)
v = ID2I(1,20,v);                  % v = (I(1) kron D2 kron I(20)) * v
v = ID2I(3,10,v);                  % v = (I(3) kron D2 kron I(10)) * v
v = ID2I(9,5,v);                   % v = (I(9) kron D2 kron I(5)) * v
v = v.*u(132:266);                      
v = ID2tI(1,45,v);                 % v = (I(1) kron D2' kron I(45)) * v
v = ID2tI(2,15,v);                 % v = (I(2) kron D2' kron I(15)) * v
v = ID2tI(4,5,v);                  % v = (I(4) kron D2' kron I(5)) * v
y(50:73) = ID3tI(8,1,v);           % y(50:73) = (I(8) kron D3' kron I(1)) * v
% --------------------------------------------------------------------------------
y(2) = y(1)+y(2);                                 % DC term calculation
y(2:73) = tKRED([2,3],[3,2],2,y(2:73));           % transpose reduction operations
y = y(op);                                        % output permutation

% For complex data - 
% Total Number of Real Multiplications : 532
% Total Number of Real Additions: 2504

